@@ -8,9 +8,10 @@ from seesaw.config import realize
 from tornado import ioloop
 
 class Runner(object):
-  def __init__(self, pipeline, stop_file=None, concurrent_items=1):
-    self.pipeline = pipeline
+  def __init__(self, stop_file=None, concurrent_items=1, max_items=None):
+    self.pipeline = None
     self.concurrent_items = concurrent_items
+    self.max_items = max_items
 
     self.item_count = 0
     self.active_items = set()
@@ -20,12 +21,29 @@ class Runner(object):
 
     self.on_status = Event()
     self.on_create_item = Event()
+    self.on_pipeline_start_item = Event()
+    self.on_pipeline_finish_item = Event()
     self.on_finish = Event()
-    self.pipeline.on_finish_item += self._item_finished
 
     if stop_file:
       ioloop.PeriodicCallback(self.check_stop_file, 5000).start()
 
+  def set_current_pipeline(self, pipeline):
+    old_pipeline = self.pipeline
+
+    if pipeline:
+      pipeline.on_start_item += self._item_starting
+      pipeline.on_finish_item += self._item_finished
+
+    self.pipeline = pipeline
+
+    if old_pipeline:
+      # stop any cancellable items in the previous pipeline
+      old_pipeline.cancel_items()
+
+  def is_active(self):
+    return len(self.active_items) > 0
+
   def start(self):
     self.add_items()
 
@@ -63,26 +81,35 @@ class Runner(object):
       return None
 
   def add_items(self):
-    items_required = int(realize(self.concurrent_items))
-    while len(self.active_items) < items_required:
-      self.item_count += 1
-      item_id = "%d-%d" % (id(self), self.item_count)
-      item = Item(pipeline=self.pipeline, item_id=item_id, item_number=self.item_count)
-      self.on_create_item(self, item)
-      self.active_items.add(item)
-      self.pipeline.enqueue(item)
+    if self.pipeline:
+      if self.max_items and self.max_items <= self.item_count:
+        return
+
+      items_required = int(realize(self.concurrent_items))
+      while len(self.active_items) < items_required:
+        self.item_count += 1
+        item_id = "%d-%d" % (id(self), self.item_count)
+        item = Item(pipeline=self.pipeline, item_id=item_id, item_number=self.item_count)
+        self.on_create_item(self, item)
+        self.active_items.add(item)
+        self.pipeline.enqueue(item)
+
+  def _item_starting(self, pipeline, item):
+    self.on_pipeline_start_item(self, pipeline, item)
 
   def _item_finished(self, pipeline, item):
+    self.on_pipeline_finish_item(self, pipeline, item)
     self.active_items.remove(item)
     if not self.should_stop():
       self.add_items()
-    elif len(self.active_items) == 0:
+    if len(self.active_items) == 0:
       self.on_finish(self)
 
 class SimpleRunner(Runner):
-  def __init__(self, pipeline, stop_file=None, concurrent_items=1):
-    Runner.__init__(self, pipeline, stop_file=stop_file, concurrent_items=concurrent_items)
+  def __init__(self, pipeline, stop_file=None, concurrent_items=1, max_items=None):
+    Runner.__init__(self, stop_file=stop_file, concurrent_items=concurrent_items, max_items=max_items)
 
+    self.set_current_pipeline(pipeline)
     self.on_create_item += self._handle_create_item
     self.on_finish += self._stop_ioloop
 
