@@ -1,5 +1,6 @@
 import re
 import os.path
+import time
 
 from tornado import web, ioloop, template
 from tornadio2 import SocketConnection, TornadioRouter, SocketServer, event
@@ -11,12 +12,12 @@ TEMPLATES_PATH = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__
 
 class IndexHandler(web.RequestHandler):
   def get(self):
-    self.render(os.path.join(PUBLIC_PATH, "index.html"))
+    self.render(os.path.join(PUBLIC_PATH, "index.html"), timestamp=time.time())
 
 
 class ItemMonitor(object):
-  def __init__(self, pipeline, item):
-    self.pipeline = pipeline
+  def __init__(self, item):
+    self.pipeline = item.pipeline
     self.item = item
 
     item.on_output += self.handle_item_output
@@ -63,7 +64,6 @@ class ItemMonitor(object):
       return "running"
 
   def handle_item_output(self, item, data):
-    data = re.sub("\r\n?", "\n", data)
     self.collected_data.append(data)
     SeesawConnection.broadcast("item.output", { "item_id": item.item_id, "data": data })
 
@@ -93,9 +93,6 @@ class ApiHandler(web.RequestHandler):
     return TEMPLATES_PATH
 
   def post(self, command):
-    if self.warrior:
-      self.runner = self.warrior.current_runner
-
     if command == "stop":
       if self.warrior:
         self.warrior.stop_gracefully()
@@ -207,10 +204,6 @@ class SeesawConnection(SocketConnection):
   def handle_project_refresh(cls, warrior, project, runner):
     cls.project = project
     cls.runner = runner
-    if project:
-      runner.pipeline.on_start_item += SeesawConnection.handle_start_item
-      runner.pipeline.on_finish_item += SeesawConnection.handle_finish_item
-      runner.on_status += SeesawConnection.handle_runner_status
     cls.broadcast_project_refresh()
 
   @classmethod
@@ -231,11 +224,11 @@ class SeesawConnection(SocketConnection):
     })
 
   @classmethod
-  def handle_start_item(cls, pipeline, item):
-    cls.item_monitors[item] = ItemMonitor(pipeline, item)
+  def handle_start_item(cls, runner, pipeline, item):
+    cls.item_monitors[item] = ItemMonitor(item)
 
   @classmethod
-  def handle_finish_item(cls, pipeline, item):
+  def handle_finish_item(cls, runner, pipeline, item):
     del cls.item_monitors[item]
 
   @classmethod
@@ -257,8 +250,8 @@ def start_runner_server(project, runner, bind_address="", port_number=8001):
   SeesawConnection.project = project
   SeesawConnection.runner = runner
 
-  runner.pipeline.on_start_item += SeesawConnection.handle_start_item
-  runner.pipeline.on_finish_item += SeesawConnection.handle_finish_item
+  runner.on_pipeline_start_item += SeesawConnection.handle_start_item
+  runner.on_pipeline_finish_item += SeesawConnection.handle_finish_item
   runner.on_status += SeesawConnection.handle_runner_status
 
   router = TornadioRouter(SeesawConnection)
@@ -284,6 +277,9 @@ def start_warrior_server(warrior, port_number=8001):
   warrior.on_project_installation_failed += SeesawConnection.handle_project_installation_failed
   warrior.on_project_selected += SeesawConnection.handle_project_selected
   warrior.on_status += SeesawConnection.handle_warrior_status
+  warrior.runner.on_pipeline_start_item += SeesawConnection.handle_start_item
+  warrior.runner.on_pipeline_finish_item += SeesawConnection.handle_finish_item
+  warrior.runner.on_status += SeesawConnection.handle_runner_status
 
   ioloop.PeriodicCallback(SeesawConnection.broadcast_bandwidth, 1000).start()
 
